(function (factory) {
    if (typeof module === 'object' && typeof module.exports === 'object') {
        var v = factory(require, exports); if (v !== undefined) module.exports = v;
    }
    else if (typeof define === 'function' && define.amd) {
        define(["require", "exports", '../parser/cssNodes', '../parser/cssSymbolScope', './languageFacts', '../utils/strings'], factory);
    }
})(function (require, exports) {
    /*---------------------------------------------------------------------------------------------
     *  Copyright (c) Microsoft Corporation. All rights reserved.
     *  Licensed under the MIT License. See License.txt in the project root for license information.
     *--------------------------------------------------------------------------------------------*/
    'use strict';
    var nodes = require('../parser/cssNodes');
    var cssSymbolScope_1 = require('../parser/cssSymbolScope');
    var languageFacts = require('./languageFacts');
    var strings = require('../utils/strings');
    var CSSCompletion = (function () {
        function CSSCompletion(variablePrefix) {
            if (variablePrefix === void 0) { variablePrefix = null; }
            this.variablePrefix = variablePrefix;
        }
        CSSCompletion.prototype.getSymbolContext = function () {
            if (!this.symbolContext) {
                this.symbolContext = new cssSymbolScope_1.Symbols(this.styleSheet);
            }
            return this.symbolContext;
        };
        CSSCompletion.prototype.doComplete = function (document, position, styleSheet) {
            this.offset = document.offsetAt(position);
            this.position = position;
            this.currentWord = getCurrentWord(document, this.offset);
            this.textDocument = document;
            this.styleSheet = styleSheet;
            var result = { isIncomplete: false, items: [] };
            var nodepath = nodes.getNodePath(this.styleSheet, this.offset);
            for (var i = nodepath.length - 1; i >= 0; i--) {
                var node = nodepath[i];
                if (node instanceof nodes.Property) {
                    this.getCompletionsForDeclarationProperty(result);
                }
                else if (node instanceof nodes.Expression) {
                    this.getCompletionsForExpression(node, result);
                }
                else if (node instanceof nodes.SimpleSelector) {
                    var parentRuleSet = node.findParent(nodes.NodeType.Ruleset);
                    this.getCompletionsForSelector(parentRuleSet, result);
                }
                else if (node instanceof nodes.Declarations) {
                    this.getCompletionsForDeclarations(node, result);
                }
                else if (node instanceof nodes.VariableDeclaration) {
                    this.getCompletionsForVariableDeclaration(node, result);
                }
                else if (node instanceof nodes.RuleSet) {
                    this.getCompletionsForRuleSet(node, result);
                }
                else if (node instanceof nodes.Interpolation) {
                    this.getCompletionsForInterpolation(node, result);
                }
                else if (node instanceof nodes.FunctionArgument) {
                    this.getCompletionsForFunctionArgument(node, node.getParent(), result);
                }
                else if (node instanceof nodes.FunctionDeclaration) {
                    this.getCompletionsForFunctionDeclaration(node, result);
                }
                else if (node instanceof nodes.Function) {
                    this.getCompletionsForFunctionArgument(null, node, result);
                }
                if (result.items.length > 0) {
                    return result;
                }
            }
            this.getCompletionsForStylesheet(result);
            if (result.items.length > 0) {
                return result;
            }
            if (this.variablePrefix && this.currentWord.indexOf(this.variablePrefix) === 0) {
                this.getVariableProposals(result);
                if (result.items.length > 0) {
                    return result;
                }
            }
            // no match, don't show text matches
            return result;
        };
        CSSCompletion.prototype.getCompletionsForDeclarationProperty = function (result) {
            return this.getPropertyProposals(result);
        };
        CSSCompletion.prototype.getPropertyProposals = function (result) {
            var properties = languageFacts.getProperties();
            for (var key in properties) {
                if (properties.hasOwnProperty(key)) {
                    var entry = properties[key];
                    if (entry.browsers.onCodeComplete) {
                        result.items.push({
                            label: entry.name,
                            documentation: languageFacts.getEntryDescription(entry),
                            insertText: entry.name + ': ',
                            kind: 10 /* Property */
                        });
                    }
                }
            }
            return result;
        };
        CSSCompletion.prototype.getCompletionsForDeclarationValue = function (node, result) {
            var propertyName = node.getFullPropertyName();
            var entry = languageFacts.getProperties()[propertyName];
            if (entry) {
                this.getColorProposals(entry, result);
                this.getPositionProposals(entry, result);
                this.getRepeatStyleProposals(entry, result);
                this.getLineProposals(entry, result);
                this.getBoxProposals(entry, result);
                this.getImageProposals(entry, result);
                this.getTimingFunctionProposals(entry, result);
                this.getBasicShapeProposals(entry, result);
                this.getValueEnumProposals(entry, result);
                this.getCSSWideKeywordProposals(entry, result);
                this.getUnitProposals(entry, result);
            }
            else {
                var existingValues = new Set();
                this.styleSheet.accept(new ValuesCollector(propertyName, existingValues));
                existingValues.getEntries().forEach(function (existingValue) {
                    result.items.push({
                        label: existingValue,
                        insertText: existingValue,
                        kind: 12 /* Value */
                    });
                });
            }
            this.getVariableProposals(result);
            this.getTermProposals(result);
            return result;
        };
        CSSCompletion.prototype.getValueEnumProposals = function (entry, result) {
            if (entry.values) {
                entry.values.forEach(function (value) {
                    if (languageFacts.isCommonValue(value)) {
                        result.items.push({
                            label: value.name,
                            documentation: languageFacts.getEntryDescription(value),
                            insertText: value.name,
                            kind: 12 /* Value */
                        });
                    }
                });
            }
            return result;
        };
        CSSCompletion.prototype.getCSSWideKeywordProposals = function (entry, result) {
            for (var keywords in languageFacts.cssWideKeywords) {
                result.items.push({
                    label: keywords,
                    documentation: languageFacts.cssWideKeywords[keywords],
                    insertText: keywords,
                    kind: 12 /* Value */
                });
            }
            return result;
        };
        CSSCompletion.prototype.getCompletionsForInterpolation = function (node, result) {
            if (this.offset >= node.offset + 2) {
                this.getVariableProposals(result);
            }
            return result;
        };
        CSSCompletion.prototype.getVariableProposals = function (result) {
            var symbols = this.getSymbolContext().findSymbolsAtOffset(this.offset, nodes.ReferenceType.Variable);
            symbols.forEach(function (symbol) {
                result.items.push({
                    label: symbol.name,
                    insertText: strings.startsWith(symbol.name, '--') ? "var(" + symbol.name + ")" : symbol.name,
                    kind: 6 /* Variable */
                });
            });
            return result;
        };
        CSSCompletion.prototype.getVariableProposalsForCSSVarFunction = function (result) {
            var symbols = this.getSymbolContext().findSymbolsAtOffset(this.offset, nodes.ReferenceType.Variable);
            symbols = symbols.filter(function (symbol) {
                return strings.startsWith(symbol.name, '--');
            });
            symbols.forEach(function (symbol) {
                result.items.push({
                    label: symbol.name,
                    insertText: symbol.name,
                    kind: 6 /* Variable */
                });
            });
            return result;
        };
        CSSCompletion.prototype.getUnitProposals = function (entry, result) {
            var currentWord = '0';
            if (this.currentWord.length > 0) {
                var numMatch = this.currentWord.match(/^-?\d[\.\d+]*/);
                if (numMatch) {
                    currentWord = numMatch[0];
                    result.isIncomplete = currentWord.length === this.currentWord.length;
                }
            }
            else if (this.currentWord.length === 0) {
                result.isIncomplete = true;
            }
            entry.restrictions.forEach(function (restriction) {
                var units = languageFacts.units[restriction];
                if (units) {
                    units.forEach(function (unit) {
                        result.items.push({
                            label: currentWord + unit,
                            insertText: currentWord + unit,
                            kind: 11 /* Unit */
                        });
                    });
                }
            });
            return result;
        };
        CSSCompletion.prototype.getColorProposals = function (entry, result) {
            if (entry.restrictions.indexOf('color') !== -1) {
                for (var color in languageFacts.colors) {
                    result.items.push({
                        label: color,
                        documentation: languageFacts.colors[color],
                        insertText: color,
                        kind: 16 /* Color */
                    });
                }
                for (var color in languageFacts.colorKeywords) {
                    result.items.push({
                        label: color,
                        documentation: languageFacts.colorKeywords[color],
                        insertText: color,
                        kind: 12 /* Value */
                    });
                }
                var colorValues = new Set();
                this.styleSheet.accept(new ColorValueCollector(colorValues));
                colorValues.getEntries().forEach(function (color) {
                    result.items.push({
                        label: color,
                        insertText: color,
                        kind: 16 /* Color */
                    });
                });
                languageFacts.colorFunctions.forEach(function (p) {
                    result.items.push({
                        label: p.func.substr(0, p.func.indexOf('(')),
                        detail: p.func,
                        documentation: p.desc,
                        insertText: p.func.replace(/\[?\$(\w+)\]?/g, '{{$1}}'),
                        kind: 3 /* Function */
                    });
                });
            }
            return result;
        };
        CSSCompletion.prototype.getPositionProposals = function (entry, result) {
            if (entry.restrictions.indexOf('position') !== -1) {
                for (var position in languageFacts.positionKeywords) {
                    result.items.push({
                        label: position,
                        documentation: languageFacts.positionKeywords[position],
                        insertText: position,
                        kind: 12 /* Value */
                    });
                }
            }
            return result;
        };
        CSSCompletion.prototype.getRepeatStyleProposals = function (entry, result) {
            if (entry.restrictions.indexOf('repeat') !== -1) {
                for (var repeat in languageFacts.repeatStyleKeywords) {
                    result.items.push({
                        label: repeat,
                        documentation: languageFacts.repeatStyleKeywords[repeat],
                        insertText: repeat,
                        kind: 12 /* Value */
                    });
                }
            }
            return result;
        };
        CSSCompletion.prototype.getLineProposals = function (entry, result) {
            if (entry.restrictions.indexOf('line-style') !== -1) {
                for (var lineStyle in languageFacts.lineStyleKeywords) {
                    result.items.push({
                        label: lineStyle,
                        documentation: languageFacts.lineStyleKeywords[lineStyle],
                        insertText: lineStyle,
                        kind: 12 /* Value */
                    });
                }
            }
            if (entry.restrictions.indexOf('line-width') !== -1) {
                languageFacts.lineWidthKeywords.forEach(function (lineWidth) {
                    result.items.push({
                        label: lineWidth,
                        insertText: lineWidth,
                        kind: 12 /* Value */
                    });
                });
            }
            return result;
        };
        CSSCompletion.prototype.getBoxProposals = function (entry, result) {
            var geometryBox = entry.restrictions.indexOf('geometry-box');
            if (geometryBox !== -1) {
                for (var box in languageFacts.geometryBoxKeywords) {
                    result.items.push({
                        label: box,
                        documentation: languageFacts.geometryBoxKeywords[box],
                        insertText: box,
                        kind: 12 /* Value */
                    });
                }
            }
            if (entry.restrictions.indexOf('box') !== -1 || geometryBox !== -1) {
                for (var box in languageFacts.boxKeywords) {
                    result.items.push({
                        label: box,
                        documentation: languageFacts.boxKeywords[box],
                        insertText: box,
                        kind: 12 /* Value */
                    });
                }
            }
            return result;
        };
        CSSCompletion.prototype.getImageProposals = function (entry, result) {
            if (entry.restrictions.indexOf('image') !== -1) {
                for (var image in languageFacts.imageFunctions) {
                    result.items.push({
                        label: image,
                        documentation: languageFacts.imageFunctions[image],
                        insertText: image,
                        kind: 3 /* Function */
                    });
                }
            }
            return result;
        };
        CSSCompletion.prototype.getTimingFunctionProposals = function (entry, result) {
            if (entry.restrictions.indexOf('timing-function') !== -1) {
                for (var timing in languageFacts.transitionTimingFunctions) {
                    result.items.push({
                        label: timing,
                        documentation: languageFacts.transitionTimingFunctions[timing],
                        insertText: timing,
                        kind: 3 /* Function */
                    });
                }
            }
            return result;
        };
        CSSCompletion.prototype.getBasicShapeProposals = function (entry, result) {
            if (entry.restrictions.indexOf('shape') !== -1) {
                for (var shape in languageFacts.basicShapeFunctions) {
                    result.items.push({
                        label: shape,
                        documentation: languageFacts.basicShapeFunctions[shape],
                        insertText: shape,
                        kind: 3 /* Function */
                    });
                }
            }
            return result;
        };
        CSSCompletion.prototype.getCompletionsForStylesheet = function (result) {
            var node = this.styleSheet.findFirstChildBeforeOffset(this.offset);
            if (!node) {
                return this.getCompletionForTopLevel(result);
            }
            if (node instanceof nodes.RuleSet) {
                return this.getCompletionsForRuleSet(node, result);
            }
            return result;
        };
        CSSCompletion.prototype.getCompletionForTopLevel = function (result) {
            languageFacts.getAtDirectives().forEach(function (entry) {
                if (entry.browsers.count > 0) {
                    result.items.push({
                        label: entry.name,
                        insertText: entry.name,
                        documentation: languageFacts.getEntryDescription(entry),
                        kind: 14 /* Keyword */
                    });
                }
            });
            this.getCompletionsForSelector(null, result);
            return result;
        };
        CSSCompletion.prototype.getCompletionsForRuleSet = function (ruleSet, result) {
            var declarations = ruleSet.getDeclarations();
            var isAfter = declarations && declarations.endsWith('}') && this.offset >= declarations.end;
            if (isAfter) {
                return this.getCompletionForTopLevel(result);
            }
            var isInSelectors = !declarations || this.offset <= declarations.offset;
            if (isInSelectors) {
                return this.getCompletionsForSelector(ruleSet, result);
            }
            ruleSet.findParent(nodes.NodeType.Ruleset);
            return this.getCompletionsForDeclarations(ruleSet.getDeclarations(), result);
        };
        CSSCompletion.prototype.getCompletionsForSelector = function (ruleSet, result) {
            languageFacts.getPseudoClasses().forEach(function (entry) {
                if (entry.browsers.onCodeComplete) {
                    result.items.push({
                        label: entry.name,
                        insertText: entry.name,
                        documentation: languageFacts.getEntryDescription(entry),
                        kind: 3 /* Function */
                    });
                }
            });
            languageFacts.getPseudoElements().forEach(function (entry) {
                if (entry.browsers.onCodeComplete) {
                    result.items.push({
                        label: entry.name,
                        insertText: entry.name,
                        documentation: languageFacts.getEntryDescription(entry),
                        kind: 3 /* Function */
                    });
                }
            });
            languageFacts.html5Tags.forEach(function (entry) {
                result.items.push({
                    label: entry,
                    insertText: entry,
                    kind: 14 /* Keyword */
                });
            });
            languageFacts.svgElements.forEach(function (entry) {
                result.items.push({
                    label: entry,
                    insertText: entry,
                    kind: 14 /* Keyword */
                });
            });
            var visited = {};
            visited[this.currentWord] = true;
            var textProvider = this.styleSheet.getTextProvider();
            this.styleSheet.accept(function (n) {
                if (n.type === nodes.NodeType.SimpleSelector && n.length > 0) {
                    var selector = textProvider(n.offset, n.length);
                    if (selector.charAt(0) === '.' && !visited[selector]) {
                        visited[selector] = true;
                        result.items.push({
                            label: selector,
                            insertText: selector,
                            kind: 14 /* Keyword */
                        });
                    }
                    return false;
                }
                return true;
            });
            if (ruleSet && ruleSet.isNested()) {
                var selector = ruleSet.getSelectors().findFirstChildBeforeOffset(this.offset);
                if (selector && ruleSet.getSelectors().getChildren().indexOf(selector) === 0) {
                    this.getPropertyProposals(result);
                }
            }
            return result;
        };
        CSSCompletion.prototype.getCompletionsForDeclarations = function (declarations, result) {
            if (!declarations) {
                return result;
            }
            var node = declarations.findFirstChildBeforeOffset(this.offset);
            if (!node) {
                return this.getCompletionsForDeclarationProperty(result);
            }
            if (node instanceof nodes.AbstractDeclaration) {
                var declaration = node;
                if ((!isDefined(declaration.colonPosition) || this.offset <= declaration.colonPosition) || (isDefined(declaration.semicolonPosition) && declaration.semicolonPosition < this.offset)) {
                    if (this.offset === declaration.semicolonPosition + 1) {
                        return result; // don't show new properties right after semicolon (see Bug 15421:[intellisense] [css] Be less aggressive when manually typing CSS)
                    }
                    // complete property
                    return this.getCompletionsForDeclarationProperty(result);
                }
                if (declaration instanceof nodes.Declaration) {
                    // complete value
                    return this.getCompletionsForDeclarationValue(declaration, result);
                }
            }
            return result;
        };
        CSSCompletion.prototype.getCompletionsForVariableDeclaration = function (declaration, result) {
            if (this.offset > declaration.colonPosition) {
                this.getVariableProposals(result);
            }
            return result;
        };
        CSSCompletion.prototype.getCompletionsForExpression = function (expression, result) {
            if (expression.getParent() instanceof nodes.FunctionArgument) {
                this.getCompletionsForFunctionArgument(expression.getParent(), expression.getParent().getParent(), result);
                return result;
            }
            var declaration = expression.findParent(nodes.NodeType.Declaration);
            if (!declaration) {
                this.getTermProposals(result);
                return result;
            }
            var node = expression.findChildAtOffset(this.offset, true);
            if (!node) {
                return this.getCompletionsForDeclarationValue(declaration, result);
            }
            if (node instanceof nodes.NumericValue || node instanceof nodes.Identifier) {
                return this.getCompletionsForDeclarationValue(declaration, result);
            }
            return result;
        };
        CSSCompletion.prototype.getCompletionsForFunctionArgument = function (arg, func, result) {
            if (func.getIdentifier().getText() === 'var') {
                if (!func.getArguments().hasChildren() || func.getArguments().getChild(0) === arg) {
                    this.getVariableProposalsForCSSVarFunction(result);
                }
            }
            return result;
        };
        CSSCompletion.prototype.getCompletionsForFunctionDeclaration = function (decl, result) {
            var declarations = decl.getDeclarations();
            if (declarations && this.offset > declarations.offset && this.offset < declarations.end) {
                this.getTermProposals(result);
            }
            return result;
        };
        CSSCompletion.prototype.getTermProposals = function (result) {
            var allFunctions = this.getSymbolContext().findSymbolsAtOffset(this.offset, nodes.ReferenceType.Function);
            allFunctions.forEach(function (functionSymbol) {
                if (functionSymbol.node instanceof nodes.FunctionDeclaration) {
                    var functionDecl = functionSymbol.node;
                    var params = functionDecl.getParameters().getChildren().map(function (c) {
                        return (c instanceof nodes.FunctionParameter) ? c.getName() : c.getText();
                    });
                    result.items.push({
                        label: functionSymbol.name,
                        detail: functionSymbol.name + '(' + params.join(', ') + ')',
                        insertText: functionSymbol.name + '(' + params.map(function (p) { return '{{' + p + '}}'; }).join(', ') + ')',
                        kind: 3 /* Function */
                    });
                }
            });
            return result;
        };
        return CSSCompletion;
    }());
    exports.CSSCompletion = CSSCompletion;
    var Set = (function () {
        function Set() {
            this.entries = {};
        }
        Set.prototype.add = function (entry) {
            this.entries[entry] = true;
        };
        Set.prototype.getEntries = function () {
            return Object.keys(this.entries);
        };
        return Set;
    }());
    var InternalValueCollector = (function () {
        function InternalValueCollector(entries) {
            this.entries = entries;
            // nothing to do
        }
        InternalValueCollector.prototype.visitNode = function (node) {
            if (node instanceof nodes.Identifier || node instanceof nodes.NumericValue || node instanceof nodes.HexColorValue) {
                this.entries.add(node.getText());
            }
            return true;
        };
        return InternalValueCollector;
    }());
    var ValuesCollector = (function () {
        function ValuesCollector(propertyName, entries) {
            this.propertyName = propertyName;
            this.entries = entries;
            // nothing to do
        }
        ValuesCollector.prototype.matchesProperty = function (decl) {
            var propertyName = decl.getFullPropertyName();
            return this.propertyName === propertyName;
        };
        ValuesCollector.prototype.visitNode = function (node) {
            if (node instanceof nodes.Declaration) {
                if (this.matchesProperty(node)) {
                    var value = node.getValue();
                    if (value) {
                        value.accept(new InternalValueCollector(this.entries));
                    }
                }
            }
            return true;
        };
        return ValuesCollector;
    }());
    var ColorValueCollector = (function () {
        function ColorValueCollector(entries) {
            this.entries = entries;
            // nothing to do
        }
        ColorValueCollector.prototype.visitNode = function (node) {
            if (node instanceof nodes.HexColorValue || (node instanceof nodes.Function && languageFacts.isColorConstructor(node))) {
                this.entries.add(node.getText());
            }
            return true;
        };
        return ColorValueCollector;
    }());
    function isDefined(obj) {
        return typeof obj !== 'undefined';
    }
    function getCurrentWord(document, offset) {
        var i = offset - 1;
        var text = document.getText();
        while (i >= 0 && ' \t\n\r":{[,'.indexOf(text.charAt(i)) === -1) {
            i--;
        }
        return text.substring(i + 1, offset);
    }
});
//# sourceMappingURL=cssCompletion.js.map